#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.dominica import Dominica
from tests.common import CommonCountryTests


class TestDominica(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Dominica)

    def test_special_holidays(self):
        self.assertHoliday(
            "2009-07-28",
            "2009-09-03",
            "2010-01-04",
            "2019-09-19",
        )

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))

        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_carnival_monday(self):
        name = "Carnival Monday"
        self.assertHolidayName(
            name,
            "2020-02-24",
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertHolidayName(name, self.full_range)

    def test_carnival_tuesday(self):
        name = "Carnival Tuesday"
        self.assertHolidayName(
            name,
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
            "2024-02-13",
            "2025-03-04",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(self.start_year, 2010)))
        self.assertHolidayName(
            name,
            "2020-05-04",
            "2021-05-03",
            "2022-05-02",
            "2023-05-01",
            "2024-05-06",
            "2025-05-05",
        )
        self.assertHolidayName(name, range(2010, self.end_year))

        obs_dts = (
            "1994-05-02",
            "2005-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_emancipation_day(self):
        name_1990 = "First Monday of August"
        name_1998 = "Emancipation Day"

        self.assertHolidayName(
            name_1998,
            "2020-08-03",
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
            "2025-08-04",
        )
        self.assertHolidayName(name_1990, range(self.start_year, 1998))
        self.assertHolidayName(name_1998, range(1998, self.end_year))
        self.assertNoHolidayName(name_1990, range(1998, self.end_year))
        self.assertNoHolidayName(name_1998, range(self.start_year, 1998))

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-11-03" for year in self.full_range))

        obs_dts = (
            "2013-11-05",
            "2019-11-05",
            "2024-11-05",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_national_day_of_community_service(self):
        name = "National Day of Community Service"
        self.assertHolidayName(name, (f"{year}-11-04" for year in self.full_range))

        obs_dts = (
            "2007-11-05",
            "2012-11-05",
            "2018-11-05",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))

        obs_dts = (
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))

        obs_dts = (
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2010_public_holidays(self):
        # https://www.dominica-weekly.com/images/dominica-calendar-2010/1600-1280.jpg
        self.assertHolidaysInYear(
            2010,
            ("2010-01-01", "New Year's Day"),
            ("2010-01-04", "Special Public Holiday"),
            ("2010-02-15", "Carnival Monday"),
            ("2010-02-16", "Carnival Tuesday"),
            ("2010-04-02", "Good Friday"),
            ("2010-04-05", "Easter Monday"),
            ("2010-05-03", "Labour Day"),
            ("2010-05-24", "Whit Monday"),
            ("2010-08-02", "Emancipation Day"),
            ("2010-11-03", "Independence Day"),
            ("2010-11-04", "National Day of Community Service"),
            ("2010-12-25", "Christmas Day"),
            ("2010-12-26", "Boxing Day"),
            ("2010-12-27", "Boxing Day (observed)"),
        )

    def test_2011_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2011,
            ("2011-01-01", "New Year's Day"),
            ("2011-03-07", "Carnival Monday"),
            ("2011-03-08", "Carnival Tuesday"),
            ("2011-04-22", "Good Friday"),
            ("2011-04-25", "Easter Monday"),
            ("2011-05-02", "Labour Day"),
            ("2011-06-13", "Whit Monday"),
            ("2011-08-01", "Emancipation Day"),
            ("2011-11-03", "Independence Day"),
            ("2011-11-04", "National Day of Community Service"),
            ("2011-12-25", "Christmas Day"),
            ("2011-12-26", "Boxing Day"),
            ("2011-12-27", "Christmas Day (observed)"),
        )

    def test_2012_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2012,
            ("2012-01-01", "New Year's Day"),
            ("2012-01-02", "New Year's Day (observed)"),
            ("2012-02-20", "Carnival Monday"),
            ("2012-02-21", "Carnival Tuesday"),
            ("2012-04-06", "Good Friday"),
            ("2012-04-09", "Easter Monday"),
            ("2012-05-07", "Labour Day"),
            ("2012-05-28", "Whit Monday"),
            ("2012-08-06", "Emancipation Day"),
            ("2012-11-03", "Independence Day"),
            ("2012-11-04", "National Day of Community Service"),
            ("2012-11-05", "National Day of Community Service (observed)"),
            ("2012-12-25", "Christmas Day"),
            ("2012-12-26", "Boxing Day"),
        )

    def test_2013_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2013,
            ("2013-01-01", "New Year's Day"),
            ("2013-02-11", "Carnival Monday"),
            ("2013-02-12", "Carnival Tuesday"),
            ("2013-03-29", "Good Friday"),
            ("2013-04-01", "Easter Monday"),
            ("2013-05-06", "Labour Day"),
            ("2013-05-20", "Whit Monday"),
            ("2013-08-05", "Emancipation Day"),
            ("2013-11-03", "Independence Day"),
            ("2013-11-04", "National Day of Community Service"),
            ("2013-11-05", "Independence Day (observed)"),
            ("2013-12-25", "Christmas Day"),
            ("2013-12-26", "Boxing Day"),
        )

    def test_2014_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2014,
            ("2014-01-01", "New Year's Day"),
            ("2014-03-03", "Carnival Monday"),
            ("2014-03-04", "Carnival Tuesday"),
            ("2014-04-18", "Good Friday"),
            ("2014-04-21", "Easter Monday"),
            ("2014-05-05", "Labour Day"),
            ("2014-06-09", "Whit Monday"),
            ("2014-08-04", "Emancipation Day"),
            ("2014-11-03", "Independence Day"),
            ("2014-11-04", "National Day of Community Service"),
            ("2014-12-25", "Christmas Day"),
            ("2014-12-26", "Boxing Day"),
        )

    def test_2015_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2015,
            ("2015-01-01", "New Year's Day"),
            ("2015-02-16", "Carnival Monday"),
            ("2015-02-17", "Carnival Tuesday"),
            ("2015-04-03", "Good Friday"),
            ("2015-04-06", "Easter Monday"),
            ("2015-05-04", "Labour Day"),
            ("2015-05-25", "Whit Monday"),
            ("2015-08-03", "Emancipation Day"),
            ("2015-11-03", "Independence Day"),
            ("2015-11-04", "National Day of Community Service"),
            ("2015-12-25", "Christmas Day"),
            ("2015-12-26", "Boxing Day"),
        )

    def test_2016_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2016,
            ("2016-01-01", "New Year's Day"),
            ("2016-02-08", "Carnival Monday"),
            ("2016-02-09", "Carnival Tuesday"),
            ("2016-03-25", "Good Friday"),
            ("2016-03-28", "Easter Monday"),
            ("2016-05-02", "Labour Day"),
            ("2016-05-16", "Whit Monday"),
            ("2016-08-01", "Emancipation Day"),
            ("2016-11-03", "Independence Day"),
            ("2016-11-04", "National Day of Community Service"),
            ("2016-12-25", "Christmas Day"),
            ("2016-12-26", "Boxing Day"),
            ("2016-12-27", "Christmas Day (observed)"),
        )

    def test_2017_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2017,
            ("2017-01-01", "New Year's Day"),
            ("2017-01-02", "New Year's Day (observed)"),
            ("2017-02-27", "Carnival Monday"),
            ("2017-02-28", "Carnival Tuesday"),
            ("2017-04-14", "Good Friday"),
            ("2017-04-17", "Easter Monday"),
            ("2017-05-01", "Labour Day"),
            ("2017-06-05", "Whit Monday"),
            ("2017-08-07", "Emancipation Day"),
            ("2017-11-03", "Independence Day"),
            ("2017-11-04", "National Day of Community Service"),
            ("2017-12-25", "Christmas Day"),
            ("2017-12-26", "Boxing Day"),
        )

    def test_2018_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2018,
            ("2018-01-01", "New Year's Day"),
            ("2018-02-12", "Carnival Monday"),
            ("2018-02-13", "Carnival Tuesday"),
            ("2018-03-30", "Good Friday"),
            ("2018-04-02", "Easter Monday"),
            ("2018-05-07", "Labour Day"),
            ("2018-05-21", "Whit Monday"),
            ("2018-08-06", "Emancipation Day"),
            ("2018-11-03", "Independence Day"),
            ("2018-11-04", "National Day of Community Service"),
            ("2018-11-05", "National Day of Community Service (observed)"),
            ("2018-12-25", "Christmas Day"),
            ("2018-12-26", "Boxing Day"),
        )

    def test_2019_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "New Year's Day"),
            ("2019-03-04", "Carnival Monday"),
            ("2019-03-05", "Carnival Tuesday"),
            ("2019-04-19", "Good Friday"),
            ("2019-04-22", "Easter Monday"),
            ("2019-05-06", "Labour Day"),
            ("2019-06-10", "Whit Monday"),
            ("2019-08-05", "Emancipation Day"),
            ("2019-09-19", "Post-Hurricane Maria Thanksgiving Celebrations"),
            ("2019-11-03", "Independence Day"),
            ("2019-11-04", "National Day of Community Service"),
            ("2019-11-05", "Independence Day (observed)"),
            ("2019-12-25", "Christmas Day"),
            ("2019-12-26", "Boxing Day"),
        )

    def test_2020_public_holidays(self):
        # https://dominicaconsulategreece.com/dominica/public-holidays/
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-02-24", "Carnival Monday"),
            ("2020-02-25", "Carnival Tuesday"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-05-04", "Labour Day"),
            ("2020-06-01", "Whit Monday"),
            ("2020-08-03", "Emancipation Day"),
            ("2020-11-03", "Independence Day"),
            ("2020-11-04", "National Day of Community Service"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Boxing Day"),
        )

    def test_2021_public_holidays(self):
        # http://www.q95da.com/news/q95-news-received-on-december-29-2020-at-731pm-the-official-public-holiday-calendar-for-2021-approved-by-the-government-of-dominica
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "New Year's Day"),
            ("2021-02-15", "Carnival Monday"),
            ("2021-02-16", "Carnival Tuesday"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-05-03", "Labour Day"),
            ("2021-05-24", "Whit Monday"),
            ("2021-08-02", "Emancipation Day"),
            ("2021-11-03", "Independence Day"),
            ("2021-11-04", "National Day of Community Service"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-26", "Boxing Day"),
            ("2021-12-27", "Boxing Day (observed)"),
        )

    def test_2022_public_holidays(self):
        # https://dominica.gov.dm/about-dominica/public-holidays
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-02-28", "Carnival Monday"),
            ("2022-03-01", "Carnival Tuesday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-02", "Labour Day"),
            ("2022-06-06", "Whit Monday"),
            ("2022-08-01", "Emancipation Day"),
            ("2022-11-03", "Independence Day"),
            ("2022-11-04", "National Day of Community Service"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_2023_public_holidays(self):
        # https://dominica.gov.dm/about-dominica/public-holidays
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-02-20", "Carnival Monday"),
            ("2023-02-21", "Carnival Tuesday"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-01", "Labour Day"),
            ("2023-05-29", "Whit Monday"),
            ("2023-08-07", "Emancipation Day"),
            ("2023-11-03", "Independence Day"),
            ("2023-11-04", "National Day of Community Service"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_2024_public_holidays(self):
        # https://dominica.gov.dm/about-dominica/public-holidays
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-02-12", "Carnival Monday"),
            ("2024-02-13", "Carnival Tuesday"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "Labour Day"),
            ("2024-05-20", "Whit Monday"),
            ("2024-08-05", "Emancipation Day"),
            ("2024-11-03", "Independence Day"),
            ("2024-11-04", "National Day of Community Service"),
            ("2024-11-05", "Independence Day (observed)"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import HALF_DAY
from holidays.countries.curacao import Curacao
from tests.common import CommonCountryTests


class TestCuracao(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Curacao, years_half_day=range(2010, 2050))

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Curacao(categories=HALF_DAY, years=range(self.start_year, 2010)))

    def test_2016_public(self):
        self.assertHolidaysInYear(
            2016,
            ("2016-01-01", "Aña Nobo"),
            ("2016-02-08", "Dialuna despues di Carnaval Grandi"),
            ("2016-03-25", "Bièrnèsantu"),
            ("2016-03-27", "Pasku di Resurekshon"),
            ("2016-03-28", "Di dos dia di Pasku di Resurekshon"),
            ("2016-04-27", "Dia di Rey"),
            ("2016-05-02", "Dia di Obrero"),
            ("2016-05-05", "Dia di Asenshon"),
            ("2016-07-02", "Dia di Himno i Bandera"),
            ("2016-10-10", "Dia di Pais Kòrsou"),
            ("2016-12-25", "Pasku di Nasementu"),
            ("2016-12-26", "Di dos dia di Pasku di Nasementu"),
        )

    def test_2016_half_day(self):
        self.assertHalfDayHolidaysInYear(
            2016,
            ("2016-12-31", "Vispu di Aña Nobo"),
        )

    def test_new_years_day(self):
        self.assertHolidayName("Aña Nobo", (f"{year}-01-01" for year in self.full_range))

    def test_carnival_monday(self):
        name = "Dialuna despues di Carnaval Grandi"
        self.assertHolidayName(
            name,
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Bièrnèsantu"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Pasku di Resurekshon"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Di dos dia di Pasku di Resurekshon"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queens_day(self):
        name = "Dia di la Reina"
        self.assertHolidayName(
            name,
            "1961-05-01",
            "1965-04-30",
            "1967-05-01",
            "1972-05-01",
            "1978-05-01",
            "1989-04-29",
            "1995-04-29",
            "2000-04-29",
            "2006-04-29",
            "2013-04-30",
        )
        self.assertHolidayName(name, range(self.start_year, 2014))
        self.assertNoHolidayName(
            name,
            "1961-04-30",
            "1967-04-30",
            "1972-04-30",
            "1978-04-30",
            "1995-04-30",
            "1989-04-30",
            "2000-04-30",
            "2006-04-30",
        )
        self.assertNoHolidayName(name, range(2014, self.end_year))

    def test_king_day(self):
        name = "Dia di Rey"
        self.assertHolidayName(
            name,
            "2016-04-27",
            "2017-04-27",
            "2018-04-27",
            "2019-04-27",
            "2020-04-27",
            "2021-04-27",
            "2022-04-27",
            "2023-04-27",
            "2024-04-27",
            "2025-04-26",
            "2031-04-26",
            "2036-04-26",
        )
        self.assertHolidayName(name, range(2014, self.end_year))
        self.assertNoHolidayName(
            name,
            "2014-04-27",
            "2025-04-27",
            "2031-04-27",
            "2036-04-27",
        )
        self.assertNoHolidayName(name, range(self.start_year, 2014))

    def test_labor_day(self):
        name = "Dia di Obrero"
        self.assertHolidayName(
            name,
            "2016-05-02",
            "2017-05-01",
            "2018-05-01",
            "2019-05-01",
            "2020-05-01",
            "2021-05-01",
            "2022-05-02",
            "2023-05-01",
        )
        self.assertHolidayName(name, self.full_range)
        self.assertNoHolidayName(
            name,
            "2011-05-01",
            "2016-05-01",
            "2022-05-01",
        )

    def test_ascension_day(self):
        name = "Dia di Asenshon"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_sunday(self):
        name = "Domingo di Pentekòstès"
        self.assertHolidayName(
            name,
            "2005-05-15",
            "2006-06-04",
            "2007-05-27",
            "2008-05-11",
            "2009-05-31",
        )
        self.assertHolidayName(name, range(self.start_year, 2010))
        self.assertNoHolidayName(name, range(2010, self.end_year))

    def test_national_anthem_and_flag_day(self):
        name = "Dia di Himno i Bandera"
        self.assertHolidayName(name, (f"{year}-07-02" for year in range(1984, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1984))

    def test_curacao_day(self):
        name = "Dia di Pais Kòrsou"
        self.assertHolidayName(name, (f"{year}-10-10" for year in range(2010, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2010))

    def test_christmas(self):
        self.assertHolidayName("Pasku di Nasementu", (f"{year}-12-25" for year in self.full_range))
        self.assertHolidayName(
            "Di dos dia di Pasku di Nasementu", (f"{year}-12-26" for year in self.full_range)
        )

    def test_new_years_eve(self):
        name = "Vispu di Aña Nobo"
        self.assertNoHolidayName(name)
        self.assertHalfDayHolidayName(
            name, (f"{year}-12-31" for year in range(2010, self.end_year))
        )
        self.assertNoHalfDayHolidayName(name, range(self.start_year, 2010))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Aña Nobo"),
            ("2023-02-20", "Dialuna despues di Carnaval Grandi"),
            ("2023-04-07", "Bièrnèsantu"),
            ("2023-04-09", "Pasku di Resurekshon"),
            ("2023-04-10", "Di dos dia di Pasku di Resurekshon"),
            ("2023-04-27", "Dia di Rey"),
            ("2023-05-01", "Dia di Obrero"),
            ("2023-05-18", "Dia di Asenshon"),
            ("2023-07-02", "Dia di Himno i Bandera"),
            ("2023-10-10", "Dia di Pais Kòrsou"),
            ("2023-12-25", "Pasku di Nasementu"),
            ("2023-12-26", "Di dos dia di Pasku di Nasementu"),
            ("2023-12-31", "Vispu di Aña Nobo"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-02-20", "Carnival Monday"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-09", "Easter Sunday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-04-27", "King's Day"),
            ("2023-05-01", "Labor Day"),
            ("2023-05-18", "Ascension Day"),
            ("2023-07-02", "National Anthem and Flag Day"),
            ("2023-10-10", "Curaçao Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Second Day of Christmas"),
            ("2023-12-31", "New Year's Eve"),
        )

    def test_l10n_nl(self):
        self.assertLocalizedHolidays(
            "nl",
            ("2023-01-01", "Nieuwjaarsdag"),
            ("2023-02-20", "De maandag na de Grote Karnaval"),
            ("2023-04-07", "Goede Vrijdag"),
            ("2023-04-09", "Paasdag"),
            ("2023-04-10", "Tweede paasdag"),
            ("2023-04-27", "Koningsdag"),
            ("2023-05-01", "Dag van de Arbeid"),
            ("2023-05-18", "Hemelvaartsdag"),
            ("2023-07-02", "Nationale vlag en volkslied"),
            ("2023-10-10", "Dag van Land Curaçao"),
            ("2023-12-25", "Eerste Kerstdag"),
            ("2023-12-26", "Tweede kerstdag"),
            ("2023-12-31", "Oudejaarsavond"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-02-20", "Карнавальний понеділок"),
            ("2023-04-07", "Страсна пʼятниця"),
            ("2023-04-09", "Великдень"),
            ("2023-04-10", "Великодній понеділок"),
            ("2023-04-27", "День короля"),
            ("2023-05-01", "День праці"),
            ("2023-05-18", "Вознесіння Господнє"),
            ("2023-07-02", "День державного гімну та прапора"),
            ("2023-10-10", "День Кюрасао"),
            ("2023-12-25", "Різдво Христове"),
            ("2023-12-26", "Другий день Різдва"),
            ("2023-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import PUBLIC, WORKDAY
from holidays.countries.saint_kitts_and_nevis import SaintKittsAndNevis
from tests.common import CommonCountryTests


class TestSaintKittsAndNevis(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SaintKittsAndNevis)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(
            SaintKittsAndNevis(categories=WORKDAY, years=range(self.start_year, 2003))
        )

    def test_special_public_holidays(self):
        self.assertHoliday(
            "2015-02-18",
            "2017-09-20",
            "2017-12-19",
            "2022-08-08",
            "2023-07-04",
        )

    def test_special_half_day_holidays(self):
        self.assertHalfDayHoliday(
            "2017-03-23",
            "2017-04-10",
            "2018-12-31",
            "2019-12-31",
            "2022-04-27",
            "2023-07-20",
            "2023-12-30",
            "2024-08-01",
        )

    def test_carnival_day(self):
        name = "Carnival Day"
        name_lastlap = f"{name} - Last Lap"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertHolidayName(name_lastlap, (f"{year}-01-02" for year in self.full_range))
        obs_dts = (
            "1995-01-03",
            "2006-01-03",
            "2012-01-03",
            "2017-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        obs_dts_lastlap = (
            "1994-01-03",
            "2000-01-03",
            "2005-01-03",
            "2011-01-03",
            "2022-01-03",
        )
        self.assertHolidayName(f"{name_lastlap} (observed)", obs_dts_lastlap)
        self.assertNoNonObservedHoliday(obs_dts, obs_dts_lastlap)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(
            name,
            "2020-05-04",
            "2021-05-03",
            "2022-05-02",
            "2023-05-01",
            "2024-05-06",
            "2025-05-05",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Whit Monday"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_emancipation_day(self):
        name_1983 = "First Monday of August"
        name_1998 = "Emancipation Day"
        self.assertHolidayName(
            name_1998,
            "2020-08-03",
            "2021-08-02",
            "2022-08-01",
            "2023-08-07",
            "2024-08-05",
            "2025-08-04",
        )
        self.assertHolidayName(name_1983, range(self.start_year, 1998))
        self.assertHolidayName(name_1998, range(1998, self.end_year))
        self.assertNoHolidayName(name_1983, range(1998, self.end_year))
        self.assertNoHolidayName(name_1998, range(self.start_year, 1998))

    def test_culturama_day_last_lap(self):
        name = "Culturama Day - Last Lap"
        self.assertHolidayName(
            name,
            "2020-08-04",
            "2021-08-03",
            "2022-08-02",
            "2023-08-08",
            "2024-08-06",
            "2025-08-05",
        )
        self.assertHolidayName(name, self.full_range)

    def test_national_heroes_day(self):
        name = "National Heroes Day"
        self.assertHolidayName(name, (f"{year}-09-16" for year in range(1998, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1998))
        obs_dts = (
            "2001-09-17",
            "2007-09-17",
            "2012-09-17",
            "2018-09-17",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-09-19" for year in self.full_range))
        obs_dts = (
            "1993-09-20",
            "1999-09-20",
            "2004-09-20",
            "2010-09-20",
            "2021-09-20",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "1994-12-27",
            "2005-12-27",
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "1993-12-27",
            "1999-12-27",
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_kim_collins_day(self):
        name = "Kim Collins Day"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-08-25" for year in range(2003, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2003))

    def test_2015_holidays(self):  # ?
        # https://web.archive.org/web/20221102224614/https://www.gov.kn/in-skn-national-public-holidays/
        self.assertHolidays(
            SaintKittsAndNevis(categories=(PUBLIC, WORKDAY), years=2015),
            ("2015-01-01", "Carnival Day"),
            ("2015-01-02", "Carnival Day - Last Lap"),
            ("2015-02-18", "Federal Election Victory Day"),
            ("2015-04-03", "Good Friday"),
            ("2015-04-06", "Easter Monday"),
            ("2015-05-04", "Labour Day"),
            ("2015-05-25", "Whit Monday"),
            ("2015-08-03", "Emancipation Day"),
            ("2015-08-04", "Culturama Day - Last Lap"),
            ("2015-08-25", "Kim Collins Day"),
            ("2015-09-16", "National Heroes Day"),
            ("2015-09-19", "Independence Day"),
            ("2015-12-25", "Christmas Day"),
            ("2015-12-26", "Boxing Day"),
        )

    def test_2021_holidays(self):
        # https://www.facebook.com/photo/?fbid=3623684614351340
        self.assertHolidays(
            SaintKittsAndNevis(categories=(PUBLIC, WORKDAY), years=2021),
            ("2021-01-01", "Carnival Day"),
            ("2021-01-02", "Carnival Day - Last Lap"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-05-03", "Labour Day"),
            ("2021-05-24", "Whit Monday"),
            ("2021-08-02", "Emancipation Day"),
            ("2021-08-03", "Culturama Day - Last Lap"),
            ("2021-08-25", "Kim Collins Day"),
            ("2021-09-16", "National Heroes Day"),
            ("2021-09-19", "Independence Day"),
            ("2021-09-20", "Independence Day (observed)"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-26", "Boxing Day"),
            ("2021-12-27", "Boxing Day (observed)"),
        )

    def test_2022_holidays(self):
        # https://www.facebook.com/photo/?fbid=525835396250028
        self.assertHolidays(
            SaintKittsAndNevis(categories=(PUBLIC, WORKDAY), years=2022),
            ("2022-01-01", "Carnival Day"),
            ("2022-01-02", "Carnival Day - Last Lap"),
            ("2022-01-03", "Carnival Day - Last Lap (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-02", "Labour Day"),
            ("2022-06-06", "Whit Monday"),
            ("2022-08-01", "Emancipation Day"),
            ("2022-08-02", "Culturama Day - Last Lap"),
            ("2022-08-08", "Federal Election Victory Day"),
            ("2022-08-25", "Kim Collins Day"),
            ("2022-09-16", "National Heroes Day"),
            ("2022-09-19", "Independence Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_2023_holidays(self):
        # https://www.facebook.com/photo?fbid=525940556239512
        self.assertHolidays(
            SaintKittsAndNevis(categories=(PUBLIC, WORKDAY), years=2023),
            ("2023-01-01", "Carnival Day"),
            ("2023-01-02", "Carnival Day - Last Lap"),
            ("2023-01-03", "Carnival Day (observed)"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-01", "Labour Day"),
            ("2023-05-29", "Whit Monday"),
            (
                "2023-07-04",
                "50th Anniversary of the Establishment of the Caribbean Community (CARICOM)",
            ),
            ("2023-08-07", "Emancipation Day"),
            ("2023-08-08", "Culturama Day - Last Lap"),
            ("2023-09-16", "National Heroes Day"),
            ("2023-09-19", "Independence Day"),
            ("2023-08-25", "Kim Collins Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_2024_holidays(self):
        # https://www.facebook.com/photo/?fbid=769697881863777
        self.assertHolidays(
            SaintKittsAndNevis(categories=(PUBLIC, WORKDAY), years=2024),
            ("2024-01-01", "Carnival Day"),
            ("2024-01-02", "Carnival Day - Last Lap"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-06", "Labour Day"),
            ("2024-05-20", "Whit Monday"),
            ("2024-08-05", "Emancipation Day"),
            ("2024-08-06", "Culturama Day - Last Lap"),
            ("2024-09-16", "National Heroes Day"),
            ("2024-08-25", "Kim Collins Day"),
            ("2024-09-19", "Independence Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars.gregorian import AUG, SEP
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    TUE_TO_PREV_MON,
    THU_TO_NEXT_FRI,
    SUN_TO_NEXT_MON,
)


class Angola(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Angola holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Angola>
        * <https://web.archive.org/web/20240617085329/https://www.siac.gv.ao/downloads/181029-Lei-Feriados.pdf>
        * [Decree #5/75](https://web.archive.org/web/20230719141732/https://www.lexlink.eu/FileGet.aspx?FileId=3023486)
        * [Decree #92/80](https://web.archive.org/web/20250427180154/https://www.lexlink.eu/FileGet.aspx?FileId=3023473)
        * [Decree #7/92](https://web.archive.org/web/20230719141946/https://www.lexlink.eu/FileGet.aspx?FileId=3023485)
        * [Law #16/96](https://web.archive.org/web/20230719141332/https://www.lexlink.eu/FileGet.aspx?FileId=3037036)
        * [Law #1/01](https://web.archive.org/web/20230719142311/https://www.lexlink.eu/FileGet.aspx?FileId=3029035)
        * [Law #7/03](https://web.archive.org/web/20230719142347/https://www.lexlink.eu/FileGet.aspx?FileId=3002131)
        * [Law #10/11](https://web.archive.org/web/20250427180133/https://equadros.gov.ao/documents/40468/0/lei_10_11-1+(1).pdf)
        * [Law #11/18](https://web.archive.org/web/20250427175641/https://equadros.gov.ao/documents/40468/0/Lei_no_11-18+(1).pdf)
        * <https://web.archive.org/web/20250211025045/https://www.officeholidays.com/countries/angola>
        * <https://web.archive.org/web/20250323005924/https://www.timeanddate.com/holidays/angola/>
    """

    country = "AO"
    default_language = "pt_AO"
    supported_languages = ("en_US", "pt_AO", "uk")
    # %s (observed).
    observed_label = tr("%s (ponte)")
    # Decree #5/75.
    start_year = 1975

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=AngolaStaticHolidays)
        kwargs.setdefault("observed_rule", TUE_TO_PREV_MON + THU_TO_NEXT_FRI)
        super().__init__(*args, **kwargs)

    def _is_observed(self, dt: date) -> bool:
        # As per Law # 16/96, from 1996/9/27, when public holiday falls on Sunday,
        # it rolls over to the following Monday.
        return dt >= date(1996, SEP, 27)

    def _add_observed(self, dt: date, **kwargs) -> tuple[bool, date | None]:
        # As per Law # #11/18, from 2018/9/10, when public holiday falls on Tuesday or Thursday,
        # the Monday or Friday is also a holiday.
        kwargs.setdefault(
            "rule", SUN_TO_NEXT_MON if dt < date(2018, SEP, 10) else self._observed_rule
        )
        return super()._add_observed(dt, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        name = tr("Dia do Ano Novo")
        dt = self._add_new_years_day(name)
        if self._year <= 2011 or self._year >= 2018:
            self._add_observed(dt)
            self._add_observed(self._next_year_new_years_day, name=name)

        # Law #16/96.
        if 1997 <= self._year <= 2011:
            self._add_observed(
                # Martyrs of Colonial Repression Day.
                self._add_holiday_jan_4(tr("Dia dos Mártires da Repressão Colonial"))
            )

        self._add_observed(
            self._add_holiday_feb_4(
                # Beginning of the Armed Struggle for National Liberation Day.
                tr("Dia do Início da Luta Armada de Libertação Nacional")
                if self._year >= 2012
                # Beginning of the Armed Struggle Day.
                else tr("Dia do Início da Luta Armada")
            )
        )

        # Law #16/96.
        if self._year >= 1997:
            # Carnival Day.
            self._add_observed(self._add_carnival_tuesday(tr("Dia do Carnaval")))

            # International Women's Day.
            self._add_observed(self._add_womens_day(tr("Dia Internacional da Mulher")))

        # Law #11/18.
        if self._year >= 2019:
            # Southern Africa Liberation Day.
            self._add_observed(self._add_holiday_mar_23(tr("Dia da Libertação da África Austral")))

        # Law #7/03.
        if self._year >= 2003:
            # Peace and National Reconciliation Day.
            self._add_observed(self._add_holiday_apr_4(tr("Dia da Paz e Reconciliação Nacional")))

        # Law #16/96.
        if self._year >= 1997:
            # Good Friday.
            self._add_good_friday(tr("Sexta-Feira Santa"))

        # International Worker's Day.
        self._add_observed(self._add_labor_day(tr("Dia Internacional do Trabalhador")))

        # Law #1/01.
        if 2001 <= self._year <= 2010:
            # Africa Day.
            self._add_observed(self._add_africa_day(tr("Dia da África")))

        # Law #16/96.
        if 1997 <= self._year <= 2010:
            # International Children's Day.
            self._add_observed(self._add_childrens_day(tr("Dia Internacional da Criança")))

        # Decree #92/80.
        if self._year >= 1980:
            self._add_observed(
                # National Heroes' Day.
                self._add_holiday_sep_17(tr("Dia do Fundador da Nação e do Herói Nacional"))
            )

        # All Souls' Day.
        dt = self._add_all_souls_day(tr("Dia dos Finados"))
        if self._year <= 2010 or self._year >= 2018:
            self._add_observed(dt)

        self._add_observed(
            self._add_holiday_nov_11(
                # National Independence Day.
                tr("Dia da Independência Nacional")
                if self._year >= 1996
                # Independence Day.
                else tr("Dia da Independência")
            )
        )

        # Decree # 7/92.
        if self._year <= 1991:
            # Date of Founding of MPLA - Labor Party.
            self._add_holiday_dec_10(tr("Data da Fundacao do MPLA - Partido do Trabalho"))

        dt = self._add_christmas_day(
            # Christmas and Family Day.
            tr("Dia de Natal e da Família")
            if self._year >= 2011
            else (
                tr("Dia do Natal")  # Christmas Day.
                if self._year >= 1996
                else tr("Dia da Família")  # Family Day.
            )
        )
        if self._year <= 2010 or self._year >= 2018:
            self._add_observed(dt)


class AO(Angola):
    pass


class AGO(Angola):
    pass


class AngolaStaticHolidays:
    special_public_holidays = {
        # General Election Day.
        2017: (AUG, 23, tr("Dia de eleições gerais")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from __future__ import annotations

from gettext import gettext as tr
from typing import TYPE_CHECKING

from holidays.calendars.chinese import VIETNAMESE_CALENDAR
from holidays.calendars.gregorian import (
    JAN,
    FEB,
    APR,
    MAY,
    SEP,
    DEC,
    MON,
    TUE,
    WED,
    THU,
    FRI,
    SAT,
    SUN,
    _timedelta,
)
from holidays.groups import ChineseCalendarHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    ObservedRule,
    SAT_TO_PREV_WORKDAY,
    SUN_TO_NEXT_WORKDAY,
    SAT_SUN_TO_NEXT_WORKDAY,
)

if TYPE_CHECKING:
    from datetime import date

NATIONAL_DAY_RULE = ObservedRule({MON: +1, TUE: -1, WED: -1, THU: +1, FRI: -1, SAT: -1, SUN: +1})


class Vietnam(ObservedHolidayBase, ChineseCalendarHolidays, InternationalHolidays, StaticHolidays):
    """Vietnam holidays.

    References:
        * [Labor Code 1994 (Art. 73) (en)](https://web.archive.org/web/20160630110506/http://vbpl.vn/TW/Pages/vbpqen-toanvan.aspx?ItemID=2835)
        * [Labor Code 2012 (Art. 115) (en)](https://web.archive.org/web/20240706232343/http://vbpl.vn/TW/Pages/vbpqen-toanvan.aspx?ItemID=11013)
        * [Labor Code 2012 (Art. 115) (vi)](https://web.archive.org/web/20240917113133/https://vbpl.vn/TW/Pages/vbpq-toanvan.aspx?ItemID=27615)
        * [Labor Code 2019 (Art. 112) (en)](https://web.archive.org/web/20250108181808/https://vbpl.vn/TW/pages/vbpqen-toanvan.aspx?ItemID=11135)
        * [Labor Code 2019 (Art. 112) (vi)](https://web.archive.org/web/20250221171552/https://vbpl.vn/TW/Pages/vbpq-van-ban-goc.aspx?ItemID=139264)
    """

    country = "VN"
    # %s (estimated).
    estimated_label = tr("%s (dự kiến)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (nghỉ bù, dự kiến)")
    # %s (observed).
    observed_label = tr("%s (nghỉ bù)")
    default_language = "vi"
    supported_languages = ("en_US", "th", "vi")

    def __init__(self, *args, **kwargs):
        ChineseCalendarHolidays.__init__(self, calendar=VIETNAMESE_CALENDAR)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, VietnamStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        kwargs.setdefault("observed_since", 1995)
        super().__init__(*args, **kwargs)

    def _add_lunar_new_year_observed(self, dt_lny: date) -> None:
        if self._year <= 1994:
            return None

        day_names = {
            # 29 of Lunar New Year.
            -2: tr("29 Tết"),
            # Fourth Day of Lunar New Year.
            3: tr("Mùng bốn Tết Nguyên Đán"),
            # Fifth Day of Lunar New Year.
            4: tr("Mùng năm Tết Nguyên Đán"),
            # Sixth Day of Lunar New Year.
            5: tr("Mùng sáu Tết Nguyên Đán"),
        }
        for delta in range(-1, 4 if self._year >= 2013 else 3):
            dt = _timedelta(dt_lny, delta)
            dt_observed = self._get_observed_date(
                dt,
                rule=(
                    SAT_TO_PREV_WORKDAY + SUN_TO_NEXT_WORKDAY
                    if self._year >= 2014
                    else SAT_SUN_TO_NEXT_WORKDAY
                ),
            )
            if dt_observed != dt:
                self._add_holiday(
                    day_names[(dt_observed - dt_lny).days],  # type: ignore[operator]
                    dt_observed,
                )

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        dts_observed.add(self._add_new_years_day(tr("Tết Dương lịch")))

        # Lunar New Year's Eve.
        self._add_chinese_new_years_eve(tr("Giao thừa Tết Nguyên Đán"))

        # Lunar New Year.
        lny = self._add_chinese_new_years_day(tr("Tết Nguyên Đán"))

        # Second Day of Lunar New Year.
        self._add_chinese_new_years_day_two(tr("Mùng hai Tết Nguyên Đán"))

        # Third Day of Lunar New Year.
        self._add_chinese_new_years_day_three(tr("Mùng ba Tết Nguyên Đán"))

        if self._year >= 2013:
            # Fourth Day of Lunar New Year.
            self._add_chinese_new_years_day_four(tr("Mùng bốn Tết Nguyên Đán"))

        if self._year >= 2007:
            # Hung Kings' Commemoration Day.
            dts_observed.add(self._add_hung_kings_day(tr("Ngày Giỗ Tổ Hùng Vương")))

        # Liberation Day/Reunification Day.
        dts_observed.add(self._add_holiday_apr_30(tr("Ngày Chiến thắng")))

        # International Labor Day.
        dts_observed.add(self._add_labor_day(tr("Ngày Quốc tế Lao động")))

        # National Day.
        name = tr("Quốc khánh")
        dts_observed.add(sep_2 := self._add_holiday_sep_2(name))
        if self._year >= 2021:
            self._add_holiday(name, self._get_observed_date(sep_2, NATIONAL_DAY_RULE))

        if self.observed:
            self._add_lunar_new_year_observed(lny)
            self._populate_observed(dts_observed)


class VN(Vietnam):
    pass


class VNM(Vietnam):
    pass


class VietnamStaticHolidays:
    """Vietnam special holidays.

    References:
        * [2018-2019](https://web.archive.org/web/20250427182343/https://thuvienphapluat.vn/cong-van/EN/Lao-dong-Tien-luong/Official-Dispatch-6519-VPCP-KGVX-2018-national-holidays-for-public-sector-employees/387625/tieng-anh.aspx)
        * [2024](https://web.archive.org/web/20241002165957/https://thuvienphapluat.vn/cong-van/EN/Lao-dong-Tien-luong/Official-Dispatch-2450-VPCP-KGVX-2024-swap-of-working-days-during-the-Reunification-Day/606458/tieng-anh.aspx)
    """

    # Date format (see strftime() Format Codes).
    substituted_date_format = tr("%d/%m/%Y")
    # Day off (substituted from %s).
    substituted_label = tr("Ngày nghỉ (thay cho ngày %s)")

    special_public_holidays = {
        2010: (FEB, 19, FEB, 27),
        2012: (JAN, 27, FEB, 4),
        2013: (APR, 29, MAY, 4),
        2014: (
            (MAY, 2, APR, 26),
            (SEP, 1, SEP, 6),
        ),
        2015: (
            (JAN, 2, DEC, 27, 2014),
            (FEB, 16, FEB, 14),
            (APR, 29, APR, 25),
        ),
        2018: (DEC, 31, JAN, 5, 2019),
        2019: (APR, 29, MAY, 4),
        2024: (APR, 29, MAY, 4),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.italy import Italy
from tests.common import CommonCountryTests


class TestItaly(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Italy, with_subdiv_categories=True)

    def test_special_holidays(self):
        self.assertHoliday(
            "1907-07-04",
            "1910-08-10",
            "1911-03-17",
            "1920-03-14",
            "1921-09-14",
            "1926-10-04",
            "1938-05-03",
            "1946-06-11",
            "1961-03-17",
            "2011-03-17",
        )

    def test_new_years_day(self):
        name = "Capodanno"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1875, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1875))

    def test_epiphany_day(self):
        name = "Epifania"
        self.assertHolidayName(
            name,
            (
                f"{year}-01-06"
                for year in (*range(self.start_year, 1978), *range(1986, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1978, 1986))

    def test_fat_thursday(self):
        name = "Giovedì grasso"
        self.assertNoHolidayName(name)
        self.assertNoSubdivBzHolidayName(name)
        self.assertSubdivBzHalfDayHolidayName(
            name,
            "2020-02-20",
            "2021-02-11",
            "2022-02-24",
            "2023-02-16",
            "2024-02-08",
            "2025-02-27",
        )
        self.assertSubdivBzHalfDayHolidayName(name, range(1993, self.end_year))
        self.assertNoSubdivBzHalfDayHolidayName(name, range(self.start_year, 1993))

    def test_last_day_of_carnival(self):
        name = "Ultimo giorno di carnevale"
        self.assertNoHolidayName(name)
        self.assertNoSubdivBzHolidayName(name)
        self.assertSubdivBzHalfDayHolidayName(
            name,
            "2020-02-25",
            "2021-02-16",
            "2022-03-01",
            "2023-02-21",
            "2024-02-13",
            "2025-03-04",
        )
        self.assertSubdivBzHalfDayHolidayName(name, range(1993, self.end_year))
        self.assertNoSubdivBzHalfDayHolidayName(name, range(self.start_year, 1993))

    def test_saint_josephs_day(self):
        name = "San Giuseppe"
        self.assertHolidayName(name, (f"{year}-03-19" for year in range(1929, 1977)))
        self.assertNoHolidayName(name, range(self.start_year, 1929), range(1977, self.end_year))

    def test_good_friday(self):
        name = "Venerdì santo"
        self.assertNoHolidayName(name)
        self.assertNoSubdivBzHolidayName(name)
        self.assertSubdivBzHalfDayHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertSubdivBzHalfDayHolidayName(name, range(1993, self.end_year))
        self.assertNoSubdivBzHalfDayHolidayName(name, range(self.start_year, 1993))

    def test_easter_sunday(self):
        name = "Pasqua"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Lunedì dell'Angelo"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1950, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1950))

    def test_foundation_of_rome(self):
        name = "Natale di Roma"
        self.assertHolidayName(name, (f"{year}-04-21" for year in range(1923, 1942)))
        self.assertNoHolidayName(name, range(self.start_year, 1923), range(1942, self.end_year))

    def test_liberation_day(self):
        name = "Anniversario della Liberazione"
        self.assertHolidayName(name, (f"{year}-04-25" for year in range(1946, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1946))

    def test_labor_day(self):
        name = "Festa del Lavoro"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1946, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1946))

    def test_anniversary_of_victory_in_europe(self):
        name = "Anniversario della Vittoria in Europa"
        self.assertHolidayName(name, (f"{year}-05-08" for year in range(1946, 1950)))
        self.assertNoHolidayName(name, range(self.start_year, 1946), range(1950, self.end_year))

    def test_anniversary_of_founding_of_empire(self):
        name = "Anniversario della fondazione dell'Impero"
        self.assertHolidayName(name, (f"{year}-05-09" for year in range(1939, 1942)))
        self.assertNoHolidayName(name, range(self.start_year, 1939), range(1942, self.end_year))

    def test_republic_day(self):
        name = "Festa della Repubblica"
        self.assertHolidayName(
            name, (f"{year}-06-02" for year in (*range(1947, 1977), *range(2001, self.end_year)))
        )
        self.assertHolidayName(
            name,
            "1977-06-05",
            "1978-06-04",
            "1997-06-01",
            "1998-06-07",
            "1999-06-06",
            "2000-06-04",
        )
        self.assertNoHolidayName(name, range(self.start_year, 1947))

    def test_whit_monday(self):
        name = "Lunedì di Pentecoste"
        self.assertNoHolidayName(name)
        self.assertSubdivBzHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertSubdivBzHolidayName(name, range(1993, self.end_year))
        self.assertNoSubdivBzHolidayName(name, range(self.start_year, 1993))

    def test_ascension_day(self):
        name = "Ascensione"
        self.assertHolidayName(
            name,
            "1971-05-20",
            "1972-05-11",
            "1973-05-31",
            "1974-05-23",
            "1975-05-08",
            "1976-05-27",
        )
        self.assertHolidayName(name, range(self.start_year, 1977))
        self.assertNoHolidayName(name, range(1977, self.end_year))

    def test_saints_peter_and_paul_day(self):
        name = "Santi Apostoli Pietro e Paolo"
        self.assertHolidayName(
            name, (f"{year}-06-29" for year in (*range(self.start_year, 1914), *range(1924, 1977)))
        )
        self.assertNoHolidayName(name, range(1914, 1924), range(1977, self.end_year))

    def test_corpus_christi_day(self):
        name = "Corpus Domini"
        self.assertHolidayName(
            name,
            "1911-06-15",
            "1912-06-06",
            "1913-05-22",
            "1971-06-10",
            "1972-06-01",
            "1973-06-21",
            "1974-06-13",
            "1975-05-29",
            "1976-06-17",
        )
        self.assertHolidayName(name, range(self.start_year, 1914), range(1924, 1977))
        self.assertNoHolidayName(name, range(1914, 1924), range(1977, self.end_year))

    def test_assumption_of_mary_day(self):
        self.assertHolidayName(
            "Assunzione della Beata Vergine Maria", (f"{year}-08-15" for year in self.full_range)
        )

    def test_nativity_of_mary_day(self):
        name = "Natività della Beata Vergine Maria"
        self.assertHolidayName(name, (f"{year}-09-08" for year in range(self.start_year, 1913)))
        self.assertNoHolidayName(name, range(1913, self.end_year))

    def test_anniversary_of_capture_of_rome(self):
        name = "Anniversario della Presa di Roma"
        self.assertHolidayName(name, (f"{year}-09-20" for year in range(1895, 1931)))
        self.assertNoHolidayName(name, range(self.start_year, 1895), range(1931, self.end_year))

    def test_saint_francis_of_assisi(self):
        name = "Festa nazionale di San Francesco d'Assisi, patrono d'Italia"
        self.assertHolidayName(name, (f"{year}-10-04" for year in range(2026, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2026))

    def test_anniversary_of_march_on_rome(self):
        name = "Anniversario della Marcia su Roma"
        self.assertHolidayName(
            name,
            "1926-10-28",
            "1927-10-30",
            "1928-10-28",
            "1929-10-27",
            (f"{year}-10-28" for year in range(1930, 1941)),
        )
        self.assertNoHolidayName(name, range(self.start_year, 1926), range(1941, self.end_year))

    def test_all_saints_day(self):
        self.assertHolidayName("Ognissanti", (f"{year}-11-01" for year in self.full_range))

    def test_national_unity_day(self):
        name_1922 = "Anniversario della Vittoria"
        name_1950 = "Giorno dell'unità nazionale"
        self.assertHolidayName(
            name_1922,
            (
                f"{year}-11-04"
                for year in (*range(1922, 1927), *range(1930, 1941), *range(1946, 1950))
            ),
            "1927-11-06",
            "1928-11-04",
            "1929-11-03",
        )
        self.assertHolidayName(name_1950, (f"{year}-11-04" for year in range(1950, 1977)))
        self.assertHolidayName(
            name_1950,
            "1977-11-06",
            "1978-11-05",
            "2020-11-01",
            "2021-11-07",
            "2022-11-06",
            "2023-11-05",
            "2024-11-03",
            "2025-11-02",
        )
        self.assertHolidayName(name_1950, range(1950, self.end_year))
        self.assertNoHolidayName(name_1922, range(1941, 1946), range(1950, self.end_year))
        self.assertNoHolidayName(name_1950, range(self.start_year, 1950))

    def test_immaculate_conception_day(self):
        name = "Immacolata Concezione"
        self.assertHolidayName(
            name,
            (
                f"{year}-12-08"
                for year in (*range(self.start_year, 1913), *range(1924, self.end_year))
            ),
        )
        self.assertNoHolidayName(name, range(1913, 1924))

    def test_christmas_eve(self):
        name = "Vigilia di Natale"
        self.assertNoHolidayName(name)
        self.assertNoSubdivBzHolidayName(name)
        self.assertSubdivBzHalfDayHolidayName(
            name, (f"{year}-12-24" for year in range(1992, self.end_year))
        )
        self.assertNoSubdivBzHalfDayHolidayName(name, range(self.start_year, 1992))

    def test_christmas_day(self):
        self.assertHolidayName("Natale", (f"{year}-12-25" for year in self.full_range))

    def test_saint_stephens_day(self):
        name = "Santo Stefano"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(1949, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1949))

    def test_last_day_of_the_year(self):
        name = "Ultimo giorno dell'anno"
        self.assertNoHolidayName(name)
        self.assertNoSubdivBzHolidayName(name)
        self.assertSubdivBzHalfDayHolidayName(
            name, (f"{year}-12-31" for year in range(1992, self.end_year))
        )
        self.assertNoSubdivBzHalfDayHolidayName(name, range(self.start_year, 1992))

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Capodanno"),
            ("2022-01-06", "Epifania"),
            ("2022-04-17", "Pasqua"),
            ("2022-04-18", "Lunedì dell'Angelo"),
            ("2022-04-25", "Anniversario della Liberazione"),
            ("2022-05-01", "Festa del Lavoro"),
            ("2022-06-02", "Festa della Repubblica"),
            ("2022-08-15", "Assunzione della Beata Vergine Maria"),
            ("2022-11-01", "Ognissanti"),
            ("2022-11-06", "Giorno dell'unità nazionale"),
            ("2022-12-08", "Immacolata Concezione"),
            ("2022-12-25", "Natale"),
            ("2022-12-26", "Santo Stefano"),
        )

    def test_no_subdiv_holidays(self):
        dt_start, dt_end = "1914-01-01", "1950-01-01"
        common_holidays_1914_1949 = self.holidays[dt_start:dt_end]
        for holidays in self.subdiv_holidays.values():
            self.assertEqual(holidays[dt_start:dt_end], common_holidays_1914_1949)

    def test_subdiv_specific_days(self):
        subdiv_holidays = {
            "AG": {"2025-02-25": "San Gerlando"},
            "AL": {"2025-11-10": "San Baudolino"},
            "AN": {"2025-05-04": "San Ciriaco"},
            "AO": {"2025-09-07": "San Grato"},
            "AP": {"2025-08-05": "Sant'Emidio"},
            "AQ": {"2025-06-10": "San Massimo d'Aveia"},
            "AR": {"2025-08-07": "San Donato d'Arezzo"},
            "AT": {"2025-05-06": "San Secondo di Asti"},
            "AV": {"2025-02-14": "San Modestino"},
            "BA": {"2025-12-06": "San Nicola"},
            "BG": {"2025-08-26": "Sant'Alessandro di Bergamo"},
            "BI": {"2025-12-26": "Santo Stefano"},
            "BL": {"2025-11-11": "San Martino"},
            "BN": {"2025-08-24": "San Bartolomeo apostolo"},
            "BO": {"2025-10-04": "San Petronio"},
            "BR": {"2025-09-07": "San Teodoro d'Amasea e San Lorenzo da Brindisi"},
            "BS": {"2025-02-15": "Santi Faustino e Giovita"},
            "BT": {
                "2025-05-03": "San Nicola Pellegrino",
                "2025-09-21": "San Riccardo di Andria",
                "2025-12-30": "San Ruggero",
            },
            "BZ": {"2025-06-09": "Lunedì di Pentecoste"},
            "CA": {"2025-10-30": "San Saturnino di Cagliari"},
            "CB": {"2025-04-23": "San Giorgio"},
            "CE": {"2025-01-20": "San Sebastiano"},
            "CH": {"2025-05-11": "San Giustino di Chieti"},
            "CL": {"2025-09-29": "San Michele Arcangelo"},
            "CN": {"2025-09-29": "San Michele Arcangelo"},
            "CO": {"2025-08-31": "Sant'Abbondio"},
            "CR": {"2025-11-13": "Sant'Omobono"},
            "CS": {"2025-02-12": "Madonna del Pilerio"},
            "CT": {"2025-02-05": "Sant'Agata"},
            "CZ": {"2025-07-16": "San Vitaliano"},
            "EN": {"2025-07-02": "Madonna della Visitazione"},
            "FC": {
                "2025-02-04": "Madonna del Fuoco",
                "2025-06-24": "San Giovanni Battista",
            },
            "FE": {"2025-04-23": "San Giorgio"},
            "FG": {"2025-03-22": "Madonna dei Sette Veli"},
            "FI": {"2025-06-24": "San Giovanni Battista"},
            "FM": {
                "2025-08-15": "Maria Santissima Assunta",
                "2025-08-16": "Maria Santissima Assunta",
            },
            "FR": {"2025-06-20": "San Silverio"},
            "GE": {"2025-06-24": "San Giovanni Battista"},
            "GO": {"2025-03-16": "Santi Ilario e Taziano"},
            "GR": {"2025-08-10": "San Lorenzo"},
            "IM": {"2025-11-26": "San Leonardo da Porto Maurizio"},
            "IS": {"2025-05-19": "San Pietro Celestino"},
            "KR": {"2025-10-09": "San Dionigi"},
            "LC": {"2025-12-06": "San Nicola"},
            "LE": {"2025-08-26": "Sant'Oronzo"},
            "LI": {"2025-05-22": "Santa Giulia"},
            "LO": {"2025-01-19": "San Bassiano"},
            "LT": {
                "2025-04-25": "San Marco Evangelista",
                "2025-07-06": "Santa Maria Goretti",
            },
            "LU": {"2025-07-12": "San Paolino di Lucca"},
            "MB": {"2025-06-24": "San Giovanni Battista"},
            "MC": {"2025-08-31": "San Giuliano l'ospitaliere"},
            "ME": {"2025-06-03": "Madonna della Lettera"},
            "MI": {"2025-12-07": "Sant'Ambrogio"},
            "MN": {"2025-03-18": "Sant'Anselmo da Baggio"},
            "MO": {"2025-01-31": "San Geminiano"},
            "MS": {"2025-10-04": "San Francesco d'Assisi"},
            "MT": {"2025-07-02": "Madonna della Bruna"},
            "NA": {"2025-09-19": "San Gennaro"},
            "NO": {"2025-01-22": "San Gaudenzio"},
            "NU": {"2025-08-05": "Nostra Signora della Neve"},
            "OR": {"2025-02-13": "Sant'Archelao"},
            "PA": {"2025-07-15": "Santa Rosalia"},
            "PC": {"2025-07-04": "Sant'Antonino di Piacenza"},
            "PD": {"2025-06-13": "Sant'Antonio di Padova"},
            "PE": {"2025-10-10": "San Cetteo"},
            "PG": {
                "2025-08-11": "Santa Chiara d'Assisi",
                "2025-10-04": "San Francesco d'Assisi",
            },
            "PI": {"2025-06-17": "San Ranieri"},
            "PN": {
                "2025-04-25": "San Marco Evangelista",
                "2025-09-08": "Madonna delle Grazie",
            },
            "PO": {"2025-12-26": "Santo Stefano"},
            "PR": {"2025-01-13": "Sant'Ilario di Poitiers"},
            "PT": {"2025-07-25": "San Jacopo"},
            "PU": {
                "2025-06-01": "San Crescentino",
                "2025-09-24": "San Terenzio di Pesaro",
            },
            "PV": {"2025-12-09": "San Siro"},
            "PZ": {"2025-05-30": "San Gerardo di Potenza"},
            "RA": {"2025-07-23": "Sant'Apollinare"},
            "RC": {"2025-04-23": "San Giorgio"},
            "RE": {"2025-11-24": "San Prospero Vescovo"},
            "RG": {
                "2025-04-23": "San Giorgio Martire",
                "2025-08-29": "San Giovanni Battista",
            },
            "RI": {"2025-12-04": "Santa Barbara"},
            "RM": {"2025-06-29": "Santi Pietro e Paolo"},
            "RN": {"2025-10-14": "San Gaudenzio"},
            "RO": {"2025-11-26": "San Bellino"},
            "SA": {"2025-09-21": "San Matteo Evangelista"},
            "SI": {"2025-12-01": "Sant'Ansano"},
            "SO": {"2025-06-19": "San Gervasio e San Protasio"},
            "SP": {"2025-03-19": "San Giuseppe"},
            "SR": {"2025-12-13": "Santa Lucia"},
            "SS": {"2025-12-06": "San Nicola"},
            "SU": {"2025-05-15": "San Ponziano"},
            "SV": {"2025-03-18": "Nostra Signora della Misericordia"},
            "TA": {"2025-05-10": "San Cataldo"},
            "TE": {"2025-12-19": "San Berardo da Pagliara"},
            "TN": {"2025-06-26": "San Vigilio"},
            "TO": {"2025-06-24": "San Giovanni Battista"},
            "TP": {"2025-08-07": "Sant'Alberto degli Abati"},
            "TR": {"2025-02-14": "San Valentino"},
            "TS": {"2025-11-03": "San Giusto"},
            "TV": {"2025-04-27": "San Liberale"},
            "UD": {"2025-07-12": "Santi Ermacora e Fortunato"},
            "VA": {"2025-05-08": "San Vittore il Moro"},
            "VB": {"2025-05-08": "San Vittore il Moro"},
            "VC": {"2025-08-01": "Sant'Eusebio di Vercelli"},
            "VE": {
                "2025-04-25": "San Marco Evangelista",
                "2025-11-21": "Madonna della Salute",
            },
            "VI": {"2025-09-08": "Madonna di Monte Berico"},
            "VR": {"2025-05-21": "San Zeno"},
            "VT": {"2025-09-04": "Santa Rosa da Viterbo"},
            "VV": {"2025-03-01": "San Leoluca"},
            "Andria": {"2025-09-21": "San Riccardo di Andria"},
            "Barletta": {"2025-12-30": "San Ruggero"},
            "Cesena": {"2025-06-24": "San Giovanni Battista"},
            "Forli": {"2025-02-04": "Madonna del Fuoco"},
            "Pesaro": {"2025-09-24": "San Terenzio di Pesaro"},
            "Trani": {"2025-05-03": "San Nicola Pellegrino"},
            "Urbino": {"2025-06-01": "San Crescentino"},
        }

        for subdiv, holidays in subdiv_holidays.items():
            for dt, name in holidays.items():
                self.assertHolidayName(name, self.subdiv_holidays[subdiv], dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Capodanno"),
            ("2025-01-06", "Epifania"),
            ("2025-01-13", "Sant'Ilario di Poitiers"),
            ("2025-01-19", "San Bassiano"),
            ("2025-01-20", "San Sebastiano"),
            ("2025-01-22", "San Gaudenzio"),
            ("2025-01-31", "San Geminiano"),
            ("2025-02-04", "Madonna del Fuoco"),
            ("2025-02-05", "Sant'Agata"),
            ("2025-02-12", "Madonna del Pilerio"),
            ("2025-02-13", "Sant'Archelao"),
            ("2025-02-14", "San Modestino; San Valentino"),
            ("2025-02-15", "Santi Faustino e Giovita"),
            ("2025-02-25", "San Gerlando"),
            ("2025-02-27", "Giovedì grasso"),
            ("2025-03-01", "San Leoluca"),
            ("2025-03-04", "Ultimo giorno di carnevale"),
            ("2025-03-16", "Santi Ilario e Taziano"),
            ("2025-03-18", "Nostra Signora della Misericordia; Sant'Anselmo da Baggio"),
            ("2025-03-19", "San Giuseppe"),
            ("2025-03-22", "Madonna dei Sette Veli"),
            ("2025-04-18", "Venerdì santo"),
            ("2025-04-20", "Pasqua"),
            ("2025-04-21", "Lunedì dell'Angelo"),
            ("2025-04-23", "San Giorgio; San Giorgio Martire"),
            ("2025-04-25", "Anniversario della Liberazione; San Marco Evangelista"),
            ("2025-04-27", "San Liberale"),
            ("2025-05-01", "Festa del Lavoro"),
            ("2025-05-03", "San Nicola Pellegrino"),
            ("2025-05-04", "San Ciriaco"),
            ("2025-05-06", "San Secondo di Asti"),
            ("2025-05-08", "San Vittore il Moro"),
            ("2025-05-10", "San Cataldo"),
            ("2025-05-11", "San Giustino di Chieti"),
            ("2025-05-15", "San Ponziano"),
            ("2025-05-19", "San Pietro Celestino"),
            ("2025-05-21", "San Zeno"),
            ("2025-05-22", "Santa Giulia"),
            ("2025-05-30", "San Gerardo di Potenza"),
            ("2025-06-01", "San Crescentino"),
            ("2025-06-02", "Festa della Repubblica"),
            ("2025-06-03", "Madonna della Lettera"),
            ("2025-06-09", "Lunedì di Pentecoste"),
            ("2025-06-10", "San Massimo d'Aveia"),
            ("2025-06-13", "Sant'Antonio di Padova"),
            ("2025-06-17", "San Ranieri"),
            ("2025-06-19", "San Gervasio e San Protasio"),
            ("2025-06-20", "San Silverio"),
            ("2025-06-24", "San Giovanni Battista"),
            ("2025-06-26", "San Vigilio"),
            ("2025-06-29", "Santi Pietro e Paolo"),
            ("2025-07-02", "Madonna della Bruna; Madonna della Visitazione"),
            ("2025-07-04", "Sant'Antonino di Piacenza"),
            ("2025-07-06", "Santa Maria Goretti"),
            ("2025-07-12", "San Paolino di Lucca; Santi Ermacora e Fortunato"),
            ("2025-07-15", "Santa Rosalia"),
            ("2025-07-16", "San Vitaliano"),
            ("2025-07-23", "Sant'Apollinare"),
            ("2025-07-25", "San Jacopo"),
            ("2025-08-01", "Sant'Eusebio di Vercelli"),
            ("2025-08-05", "Nostra Signora della Neve; Sant'Emidio"),
            ("2025-08-07", "San Donato d'Arezzo; Sant'Alberto degli Abati"),
            ("2025-08-10", "San Lorenzo"),
            ("2025-08-11", "Santa Chiara d'Assisi"),
            ("2025-08-15", "Assunzione della Beata Vergine Maria; Maria Santissima Assunta"),
            ("2025-08-16", "Maria Santissima Assunta"),
            ("2025-08-24", "San Bartolomeo apostolo"),
            ("2025-08-26", "Sant'Alessandro di Bergamo; Sant'Oronzo"),
            ("2025-08-29", "San Giovanni Battista"),
            ("2025-08-31", "San Giuliano l'ospitaliere; Sant'Abbondio"),
            ("2025-09-04", "Santa Rosa da Viterbo"),
            ("2025-09-07", "San Grato; San Teodoro d'Amasea e San Lorenzo da Brindisi"),
            ("2025-09-08", "Madonna delle Grazie; Madonna di Monte Berico"),
            ("2025-09-19", "San Gennaro"),
            ("2025-09-21", "San Matteo Evangelista; San Riccardo di Andria"),
            ("2025-09-24", "San Terenzio di Pesaro"),
            ("2025-09-29", "San Michele Arcangelo"),
            ("2025-10-04", "San Francesco d'Assisi; San Petronio"),
            ("2025-10-09", "San Dionigi"),
            ("2025-10-10", "San Cetteo"),
            ("2025-10-14", "San Gaudenzio"),
            ("2025-10-30", "San Saturnino di Cagliari"),
            ("2025-11-01", "Ognissanti"),
            ("2025-11-02", "Giorno dell'unità nazionale"),
            ("2025-11-03", "San Giusto"),
            ("2025-11-10", "San Baudolino"),
            ("2025-11-11", "San Martino"),
            ("2025-11-13", "Sant'Omobono"),
            ("2025-11-21", "Madonna della Salute"),
            ("2025-11-24", "San Prospero Vescovo"),
            ("2025-11-26", "San Bellino; San Leonardo da Porto Maurizio"),
            ("2025-12-01", "Sant'Ansano"),
            ("2025-12-04", "Santa Barbara"),
            ("2025-12-06", "San Nicola"),
            ("2025-12-07", "Sant'Ambrogio"),
            ("2025-12-08", "Immacolata Concezione"),
            ("2025-12-09", "San Siro"),
            ("2025-12-13", "Santa Lucia"),
            ("2025-12-19", "San Berardo da Pagliara"),
            ("2025-12-24", "Vigilia di Natale"),
            ("2025-12-25", "Natale"),
            ("2025-12-26", "Santo Stefano"),
            ("2025-12-30", "San Ruggero"),
            ("2025-12-31", "Ultimo giorno dell'anno"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-06", "Epiphany"),
            ("2025-01-13", "Saint Hilary of Poitiers's Day"),
            ("2025-01-19", "Saint Bassianus's Day"),
            ("2025-01-20", "Saint Sebastian's Day"),
            ("2025-01-22", "Saint Gaudentius's Day"),
            ("2025-01-31", "Saint Geminianus's Day"),
            ("2025-02-04", "Our Lady of the Fire"),
            ("2025-02-05", "Saint Agatha's Day"),
            ("2025-02-12", "Our Lady of the Pilerio"),
            ("2025-02-13", "Saint Archelaus's Day"),
            ("2025-02-14", "Saint Modestinus's Day; Saint Valentine's Day"),
            ("2025-02-15", "Saints Faustinus and Jovita's Day"),
            ("2025-02-25", "Saint Gerland's Day"),
            ("2025-02-27", "Fat Thursday"),
            ("2025-03-01", "Saint Leoluca's Day"),
            ("2025-03-04", "Last Day of Carnival"),
            ("2025-03-16", "Saints Hilary and Tatian's Day"),
            ("2025-03-18", "Our Lady of Mercy; Saint Anselm of Baggio's Day"),
            ("2025-03-19", "Saint Joseph's Day"),
            ("2025-03-22", "Our Lady of the Seven Veils"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-20", "Easter Sunday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-04-23", "Saint George's Day"),
            ("2025-04-25", "Liberation Day; Saint Mark's Day"),
            ("2025-04-27", "Saint Liberal's Day"),
            ("2025-05-01", "Labor Day"),
            ("2025-05-03", "Saint Nicholas the Pilgrim's Day"),
            ("2025-05-04", "Saint Cyriacus's Day"),
            ("2025-05-06", "Saint Secundus of Asti's Day"),
            ("2025-05-08", "Saint Victor the Moor's Day"),
            ("2025-05-10", "Saint Catald's Day"),
            ("2025-05-11", "Saint Justin of Chieti's Day"),
            ("2025-05-15", "Saint Pontian's Day"),
            ("2025-05-19", "Saint Peter Celestine's Day"),
            ("2025-05-21", "Saint Zeno's Day"),
            ("2025-05-22", "Saint Julia's Day"),
            ("2025-05-30", "Saint Gerard of Potenza's Day"),
            ("2025-06-01", "Saint Crescentinus's Day"),
            ("2025-06-02", "Republic Day"),
            ("2025-06-03", "Our Lady of the Letter"),
            ("2025-06-09", "Whit Monday"),
            ("2025-06-10", "Saint Maximus of Aveia's Day"),
            ("2025-06-13", "Saint Anthony of Padua's Day"),
            ("2025-06-17", "Saint Ranieri's Day"),
            ("2025-06-19", "Saints Gervasius and Protasius's Day"),
            ("2025-06-20", "Saint Silverius's Day"),
            ("2025-06-24", "Saint John's Day"),
            ("2025-06-26", "Saint Vigilius's Day"),
            ("2025-06-29", "Saints Peter and Paul's Day"),
            ("2025-07-02", "Our Lady of the Bruna; Visitation of Mary Day"),
            ("2025-07-04", "Saint Antoninus of Piacenza's Day"),
            ("2025-07-06", "Saint Maria Goretti's Day"),
            (
                "2025-07-12",
                "Saint Paulinus of Lucca's Day; Saints Hermagoras and Fortunatus's Day",
            ),
            ("2025-07-15", "Saint Rosalia's Day"),
            ("2025-07-16", "Saint Vitalian's Day"),
            ("2025-07-23", "Saint Apollinaris's Day"),
            ("2025-07-25", "Saint James's Day"),
            ("2025-08-01", "Saint Eusebius of Vercelli's Day"),
            ("2025-08-05", "Our Lady of the Snows; Saint Emidius's Day"),
            ("2025-08-07", "Saint Albert of Trapani's Day; Saint Donatus of Arezzo's Day"),
            ("2025-08-10", "Saint Lawrence's Day"),
            ("2025-08-11", "Saint Clare of Assisi's Day"),
            ("2025-08-15", "Assumption Day; Assumption Of Mary Day"),
            ("2025-08-16", "Assumption Day"),
            ("2025-08-24", "Saint Bartholomew's Day"),
            ("2025-08-26", "Saint Alexander of Bergamo's Day; Saint Orontius's Day"),
            ("2025-08-29", "Saint John's Day"),
            ("2025-08-31", "Saint Abbondius's Day; Saint Julian the Hospitaller's Day"),
            ("2025-09-04", "Saint Rose of Viterbo's Day"),
            (
                "2025-09-07",
                "Saint Grat's Day; Saint Theodore of Amasea and Saint Lawrence of Brindisi's Day",
            ),
            ("2025-09-08", "Our Lady of Graces; Our Lady of Monte Berico"),
            ("2025-09-19", "Saint Januarius's Day"),
            ("2025-09-21", "Saint Matthew's Day; Saint Richard of Andria's Day"),
            ("2025-09-24", "Saint Terentius of Pesaro's Day"),
            ("2025-09-29", "Saint Michael the Archangel's Day"),
            ("2025-10-04", "Saint Francis of Assisi's Day; Saint Petronius's Day"),
            ("2025-10-09", "Saint Dionysius's Day"),
            ("2025-10-10", "Saint Cetteus's Day"),
            ("2025-10-14", "Saint Gaudentius's Day"),
            ("2025-10-30", "Saint Saturninus of Cagliari's Day"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-11-02", "National Unity Day"),
            ("2025-11-03", "Saint Justus's Day"),
            ("2025-11-10", "Saint Baudolino's Day"),
            ("2025-11-11", "Saint Martin's Day"),
            ("2025-11-13", "Saint Homobonus's Day"),
            ("2025-11-21", "Our Lady of Health"),
            ("2025-11-24", "Saint Prosper of Reggio's Day"),
            ("2025-11-26", "Saint Bellinus's Day; Saint Leonard of Porto Maurizio's Day"),
            ("2025-12-01", "Saint Ansanus's Day"),
            ("2025-12-04", "Saint Barbara's Day"),
            ("2025-12-06", "Saint Nicholas's Day"),
            ("2025-12-07", "Saint Ambrose's Day"),
            ("2025-12-08", "Immaculate Conception"),
            ("2025-12-09", "Saint Syrus's Day"),
            ("2025-12-13", "Saint Lucy's Day"),
            ("2025-12-19", "Saint Berardo's Day"),
            ("2025-12-24", "Christmas Eve"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Saint Stephen's Day"),
            ("2025-12-30", "Saint Roger's Day"),
            ("2025-12-31", "Last Day of the Year"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2025-01-01", "วันขึ้นปีใหม่"),
            ("2025-01-06", "วันสมโภชพระคริสต์แสดงองค์"),
            ("2025-01-13", "วันสมโภชนักบุญฮิลารีแห่งปัวตีเย"),
            ("2025-01-19", "วันสมโภชนักบุญบัสสิอานัส"),
            ("2025-01-20", "วันสมโภชนักบุญเซบาสเตียน"),
            ("2025-01-22", "วันสมโภชนักบุญกอเดนทีอุส"),
            ("2025-01-31", "วันสมโภชนักบุญเกมีเนียนุส"),
            ("2025-02-04", "วันแม่พระแห่งไฟ"),
            ("2025-02-05", "วันสมโภชนักบุญอากาธา"),
            ("2025-02-12", "วันแม่พระแห่งปิเลรีโอ"),
            ("2025-02-13", "วันสมโภชนักบุญอาร์เคลาอุส"),
            ("2025-02-14", "วันสมโภชนักบุญวาเลนไทน์; วันสมโภชนักบุญโมเดสตินัส"),
            ("2025-02-15", "วันสมโภชนักบุญเฟาสตินัสและโยวิตา"),
            ("2025-02-25", "วันสมโภชนักบุญเกอร์ลันโด"),
            ("2025-02-27", "วันพฤหัสบดีอ้วน"),
            ("2025-03-01", "วันสมโภชนักบุญเลโอลูคา"),
            ("2025-03-04", "วันสุดท้ายของเทศกาลคาร์นิวัล"),
            ("2025-03-16", "วันสมโภชนักบุญฮิลารีและทาเทียน"),
            ("2025-03-18", "วันสมโภชนักบุญแอนเซล์มแห่งบัดโจ; วันแม่พระแห่งความเมตตา"),
            ("2025-03-19", "วันสมโภชนักบุญโยเซฟ"),
            ("2025-03-22", "วันแม่พระแห่งผ้าคลุมทั้งเจ็ด"),
            ("2025-04-18", "วันศุกร์ประเสริฐ"),
            ("2025-04-20", "วันอาทิตย์อีสเตอร์"),
            ("2025-04-21", "วันจันทร์อีสเตอร์"),
            ("2025-04-23", "วันสมโภชนักบุญจอร์จ"),
            ("2025-04-25", "วันปลดปล่อย; วันสมโภชนักบุญมาระโก ผู้นิพนธ์พระวรสาร"),
            ("2025-04-27", "วันสมโภชนักบุญลิเบอราลิสแห่งเตรวิโซ"),
            ("2025-05-01", "วันแรงงาน"),
            ("2025-05-03", "วันสมโภชนักบุญนิโคลัสผู้แสวงบุญ"),
            ("2025-05-04", "วันสมโภชนักบุญไซริอาคุส"),
            ("2025-05-06", "วันสมโภชนักบุญเซคุนดุสแห่งแอสตี้"),
            ("2025-05-08", "วันสมโภชนักบุญวิคตอร์ผู้เป็นมัวร์"),
            ("2025-05-10", "วันสมโภชนักบุญกาตัลโด"),
            ("2025-05-11", "วันสมโภชนักบุญยุสตินแห่งคีเอตี"),
            ("2025-05-15", "วันสมโภชนักบุญปอนเทียน"),
            ("2025-05-19", "วันสมโภชนักบุญเปโตร เซเลสทีน"),
            ("2025-05-21", "วันสมโภชนักบุญเซโน"),
            ("2025-05-22", "วันสมโภชนักบุญจูเลีย"),
            ("2025-05-30", "วันสมโภชนักบุญเยราร์ดแห่งโปเตนซา"),
            ("2025-06-01", "วันสมโภชนักบุญเครสเซนตีนุส"),
            ("2025-06-02", "วันสาธารณรัฐอิตาลี"),
            ("2025-06-03", "วันแม่พระแห่งราชสาส์น"),
            ("2025-06-09", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2025-06-10", "วันสมโภชนักบุญมักซีมุสแห่งอาเวอา"),
            ("2025-06-13", "วันสมโภชนักบุญอันตนแห่งปาดัว"),
            ("2025-06-17", "วันสมโภชนักบุญรานีเอรี"),
            ("2025-06-19", "วันสมโภชนักบุญเจอร์วาซีอุสและโปรตาซีอุส"),
            ("2025-06-20", "วันสมโภชนักบุญซิลเวริอุส"),
            ("2025-06-24", "วันสมโภชนักบุญยอห์นผู้ให้บัพติศมา"),
            ("2025-06-26", "วันสมโภชนักบุญวีจีลีโอ"),
            ("2025-06-29", "วันสมโภชนักบุญเปโตรและเปาโล"),
            ("2025-07-02", "วันแม่พระเสด็จเยี่ยม; วันแม่พระแห่งบรูนา"),
            ("2025-07-04", "วันสมโภชนักบุญอันโตนีนุสแห่งปิอาเชนซา"),
            ("2025-07-06", "วันสมโภชนักบุญมารีอา กอเรตตี"),
            ("2025-07-12", "วันสมโภชนักบุญเปาลีนุสแห่งลุกกา; วันสมโภชนักบุญเฮอร์มากอรัสและฟอร์จูนาทัส"),
            ("2025-07-15", "วันสมโภชนักบุญโรซาเลีย"),
            ("2025-07-16", "วันสมโภชนักบุญวิตาเลียน"),
            ("2025-07-23", "วันสมโภชนักบุญอปอลลินาริส"),
            ("2025-07-25", "วันสมโภชนักบุญยากอบ อัครสาวก"),
            ("2025-08-01", "วันสมโภชนักบุญเอวเซบิโอแห่งแวร์แชลลี"),
            ("2025-08-05", "วันสมโภชนักบุญอีมิดีอุส; วันแม่พระแห่งหิมะ"),
            ("2025-08-07", "วันสมโภชนักบุญอัลเบิร์ตแห่งทราปานี; วันสมโภชนักบุญโดนาโตแห่งอาเรซโซ"),
            ("2025-08-10", "วันสมโภชนักบุญลอเรนซ์"),
            ("2025-08-11", "วันสมโภชนักบุญคลาราแห่งอัสซีซี"),
            ("2025-08-15", "วันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2025-08-16", "วันสมโภชแม่พระรับเกียรติยกขึ้นสวรรค์"),
            ("2025-08-24", "วันสมโภชนักบุญบาร์โธโลมิว อัครสาวก"),
            ("2025-08-26", "วันสมโภชนักบุญอเล็กซานเดอร์แห่งแบร์กาโม; วันสมโภชนักบุญโอรอนเทียส"),
            ("2025-08-29", "วันสมโภชนักบุญยอห์นผู้ให้บัพติศมา"),
            ("2025-08-31", "วันสมโภชนักบุญยูเลียนผู้ใจบุญ; วันสมโภชนักบุญอับบอนดิอุส"),
            ("2025-09-04", "วันสมโภชนักบุญโรซาแห่งวิแตร์โบ"),
            (
                "2025-09-07",
                "วันสมโภชนักบุญกราตัส; วันสมโภชนักบุญธีโอดอร์แห่งอมาเซียและลอว์เรนซ์แห่งบรินดีซี",
            ),
            ("2025-09-08", "วันแม่พระแห่งพระหรรษทาน; วันแม่พระแห่งมอนเต เบริโก"),
            ("2025-09-19", "วันสมโภชนักบุญเจนนาโร"),
            ("2025-09-21", "วันสมโภชนักบุญมัทธิว อัครสาวก; วันสมโภชนักบุญริชาร์ดแห่งอันเดรีย"),
            ("2025-09-24", "วันสมโภชนักบุญเทเรนทิอัสแห่งเปซาโร"),
            ("2025-09-29", "วันฉลองอัครทูตสวรรค์ มีคาเอล"),
            ("2025-10-04", "วันสมโภชนักบุญฟรังซิสแห่งอัสซีซี; วันสมโภชนักบุญเปโตรเนียส"),
            ("2025-10-09", "วันสมโภชนักบุญไดโอนีซีอุส"),
            ("2025-10-10", "วันสมโภชนักบุญเชตเทอุส"),
            ("2025-10-14", "วันสมโภชนักบุญกอเดนทีอุส"),
            ("2025-10-30", "วันสมโภชนักบุญซาเทอร์นินัสแห่งคัลยารี"),
            ("2025-11-01", "วันสมโภชนักบุญทั้งหลาย"),
            ("2025-11-02", "วันเอกภาพแห่งชาติ"),
            ("2025-11-03", "วันสมโภชนักบุญจัสตุส"),
            ("2025-11-10", "วันสมโภชนักบุญเบาโดลิโน"),
            ("2025-11-11", "วันสมโภชนักบุญมาร์ติน"),
            ("2025-11-13", "วันสมโภชนักบุญโฮโมโบนุส"),
            ("2025-11-21", "วันแม่พระแห่งสุขภาพ"),
            ("2025-11-24", "วันสมโภชนักบุญพรอสเปอร์แห่งเรจจิโอ"),
            ("2025-11-26", "วันสมโภชนักบุญลีโอนาร์ดแห่งปอร์โต เมาริตซีโอ; วันสมโภชนักบุญเบลลินัส"),
            ("2025-12-01", "วันสมโภชนักบุญอันซานุส"),
            ("2025-12-04", "วันสมโภชนักบุญบาร์บารา"),
            ("2025-12-06", "วันสมโภชนักบุญนิโคลัส"),
            ("2025-12-07", "วันสมโภชนักบุญอัมโบรซีอุส"),
            ("2025-12-08", "วันสมโภชแม่พระผู้ปฏิสนธินิรมล"),
            ("2025-12-09", "วันสมโภชนักบุญไซรัส"),
            ("2025-12-13", "วันสมโภชนักบุญลูซี"),
            ("2025-12-19", "วันสมโภชนักบุญเบราร์โด"),
            ("2025-12-24", "วันคริสต์มาสอีฟ"),
            ("2025-12-25", "วันคริสต์มาส"),
            ("2025-12-26", "วันสมโภชนักบุญสเตเฟน"),
            ("2025-12-30", "วันสมโภชนักบุญโรเจอร์"),
            ("2025-12-31", "วันสิ้นปี"),
        )
